/**
 * EasyBeans
 * Copyright (C) 2007-2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EasyBeansDestination.java 2546 2008-02-29 13:49:31Z sauthieg $
 * --------------------------------------------------------------------------
 */
package org.ow2.easybeans.cxf.http;

import java.io.IOException;
import java.util.logging.Logger;

import org.apache.cxf.Bus;
import org.apache.cxf.message.MessageImpl;
import org.apache.cxf.service.model.EndpointInfo;
import org.apache.cxf.transport.DestinationFactory;
import org.apache.cxf.transport.servlet.ServletDestination;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Object that manages the activation/deactivation of the EJB3 endpoint.
 * @author Florent Benoit
 */
public class EasyBeansDestination extends ServletDestination {

    /**
     * Default Logger.
     */
    private static Log logger = LogFactory.getLog(EasyBeansDestination.class);

    /**
     * Logger (JDK logger used by the super classes).
     */
    private static Logger jdkLogger = Logger.getLogger(EasyBeansDestination.class.getName());

    /**
     * URLMapper.
     */
    private URLMapper mapper = null;

    /**
     * URL of this destination.
     */
    private String path = null;

    /**
     * The EasyBeans transport factory.
     */
    private DestinationFactory factory = null;

    /**
     * Build a new Destination.
     * @param bus the associated Bus
     * @param conduitInitiator the associated conduit initiator
     * @param endpointInfo the endpoint info of the destination
     * @throws IOException if super constructor fails
     */
    public EasyBeansDestination(final Bus bus,
                                final DestinationFactory conduitInitiator,
                                final EndpointInfo endpointInfo) throws IOException {
        // FIXME: 2 last parameters are null, forcing us to override shutdown() to avoid any call to that factory
        super(bus, null, endpointInfo, null, null);
        this.mapper = URLMapper.getInstance();
        this.path = endpointInfo.getAddress();
        this.factory = conduitInitiator;
    }

    /**
     * Activate receipt of incoming messages.
     */
    @Override
    protected void activate() {
        logger.debug("activate ''{0}''", this.path);
        this.mapper.addServant(this.path, this);
    }

    /**
     * Deactivate receipt of incoming messages.
     */
    @Override
    public void deactivate() {
        logger.debug("deactivate ''{0}''", this.path);
        this.mapper.removeServant(this.path, this);
    }

    /**
     * @return the JDK logger used by the super class.
     */
    @Override
    protected Logger getLogger() {
        return jdkLogger;
    }

    /**
     * Handle the given message.
     * @param inMessage the incoming message
     * @throws IOException if the handling is failing
     */
    @Override
    protected void doMessage(final MessageImpl inMessage) throws IOException {
        super.doMessage(inMessage);
    }

    /**
     * @return the EasyBeans factory.
     */
    public DestinationFactory getEasyBeansDestinationFactory() {
        return this.factory;
    }

    /**
     * @return the bus used by this Destination.
     */
    public Bus getBus() {
        return this.bus;
    }

    /**
     * Override the {@link ServletDestination} shutdown method
     * @see org.apache.cxf.transport.servlet.ServletDestination#shutdown()
     */
    @Override
    public void shutdown() {
    }

}
